import math
from Timer.Time import Time
class Vector(object):
    def __init__(self, x = 0, y = 0):
        object.__init__(self)
        self._x=x
        self._y=y
        self.oldV=(x,y)
        self.parent=None
        pass
    def Set(self, x = 0, y =0):
        self._x=x
        self._y=y
        #self.parent.rect.move_ip(x,y)
        self.oldV=(x,y)
    def SetParent(self,parent):
        self.parent=parent
        pass
    def UpadteRectPos(self):
        self.parent.rect.move_ip(self._x,self._y)
    @property
    def x(self):
        return self._x
    @property
    def y(self):
        return self._y
    @x.setter
    def x(self,value):
        self._x=value
        if self.parent!=None:
            #self.parent.rect.move_ip(value,0)
            pass
    @y.setter
    def y(self,value):
        self._y=value
        if self.parent!=None:
            #self.parent.rect.move_ip(0,value)
            pass
    @property
    def smoothX(self):
        return self._x
    @smoothX.setter
    def smoothX(self,value):
        self._x += value*Time.deltaTime
        #self.parent.rect.move_ip(value*Time.deltaTime,0)
    @property
    def smoothY(self):
        return self._x
    @smoothY.setter
    def smoothY(self,value):
        self._y += value*Time.deltaTime
        #self.parent.rect.move_ip(0,value*Time.deltaTime)
    @property
    def value(self):
        return (self._x,self._y)
    @property
    def right():
        return Vector(1,0)
    @property
    def left():
        return Vector(-1,0)
    @property
    def down():
        return Vector(0,-1)
    @property
    def up():
        return Vector(0,1)
    @property
    def zero():
        return Vector(0,0)
    @property
    def one():
        return Vector(1,1)
    @property
    def magnitude(self):
        return math.sqrt(self._x*self._x+self._y*self._y)
    @staticmethod
    def MoveTowards(current, target, maxDistanceDelta):
        if current == target:
            return target
        else:
            return (target-current)*maxDistanceDelta*Time.deltaTime
    @staticmethod
    def Move(current,maxDistanceDelta,direction):
        temp=None
        if direction == Vector.right:
            temp = Vector(current.x+maxDistanceDelta*Time.deltaTime, current.y)
            temp.parent=current.parent
            #temp.parent.rect.move_ip(maxDistanceDelta*Time.deltaTime,0)
        elif direction == Vector.left:
            temp = Vector(current.x-maxDistanceDelta*Time.deltaTime, current.y)
            temp.parent=current.parent
            #temp.parent.rect.move_ip(-maxDistanceDelta*Time.deltaTime,0)
        elif direction == Vector.up:
            temp = Vector(current.x, current.y-maxDistanceDelta*Time.deltaTime)
            temp.parent=current.parent
            #temp.parent.rect.move_ip(0,-maxDistanceDelta*Time.deltaTime)
        elif direction == Vector.down:
            temp = Vector(current.x, current.y+maxDistanceDelta*Time.deltaTime)
            temp.parent=current.parent
            #temp.parent.rect.move_ip(0,maxDistanceDelta*Time.deltaTime)
        temp.oldV=current.value
        return temp
        pass
    def __getitem__(self, key):
        pass
    def __call__(self):
        return self
    def __eq__(self, other):
        if self._x == other[0] and self._y == other[1]:
            return True
        else:
            return False
    def __sub__(self, other):
        return Vector(self._x-other.x,self._y-other.y)
    def __add__(self, other):
        return Vector(self._x+other.x,self._y+other.y)
    def __mul__(self, other: int):
        return Vector(self._x*other,self._y*other)
        


import pygame
from pygame.locals import *
from GameObject.Transform import Transform
from Engine.Engine import Engine
from Engine.Collider import Collider
from GameObject.Vector2 import Vector
import math
import numpy as np
class GameObject():
    id=0
    def __init__(self, image: str, priority: int):
        self.ColList=[]
        self.isMianCol=False
        self.isCol=False
        self.thisID=GameObject.id
        self.mycol=None
        GameObject.id+=1
        self.priority=priority
        
        self.transform=Transform(image)
        self.transform.parent=self
        self.__activation=True
        Engine.drawlist[str(priority)]=self
        Engine.drawSortList.append(priority)
        Engine.drawSortList.sort()
        pass
    def Add(self,image: str):
        self.transform.Add(image)
        pass
    def Set(self, id: int):
        if id <= self.transform.drawId and id >= 0:
            self.transform.Set(id)
        pass
    @property
    def active(self):
        return self.__activation
        pass
    def SetActive(self, active: bool):
        self.__activation=active
        pass
    def Draw(self):
        pass
    def AddComponent(self):
        pass
    @staticmethod
    def Find(name):
        pass
    def LookAt(self,obj):
        x = np.array((0,1))
        temp = (self.transform.position - obj.transform.position).value
        y = np.array(temp)
        print(y)
        # 两个向量
        Lx=np.sqrt(x.dot(x))
        Ly=np.sqrt(y.dot(y))
        #相当于勾股定理，求得斜线的长度
        cos_angle=x.dot(y)/(Lx*Ly)
        #求得cos_sita的值再反过来计算，绝对长度乘以cos角度为矢量长度
        angle3=np.arccos(cos_angle)
        angle=angle3*360/2/np.pi
        if temp[0]>0:
            angle*=-1
        self.transform.Rotate2(angle)
        #变为角度
        print(angle)
        pass
    def Move(self, speed):
        temp = self.transform.angle // 90
        #print(temp)
        if temp == 0:
            x = self.transform.angle/90
            y = 1-x
            self.transform.position=Vector.Move(self.transform.position,speed * y,Vector.left)
            self.transform.position=Vector.Move(self.transform.position,speed * x,Vector.up)
            pass
        elif temp == 1:
            temp = self.transform.angle - 90
            x = temp/90
            y = 1-x
            self.transform.position=Vector.Move(self.transform.position,speed * y,Vector.down)
            self.transform.position=Vector.Move(self.transform.position,speed * x,Vector.left)
            pass
        elif temp == 2:
            temp = self.transform.angle - 180
            x = temp/90
            y = 1-x
            self.transform.position=Vector.Move(self.transform.position,speed * y,Vector.right)
            self.transform.position=Vector.Move(self.transform.position,speed * x,Vector.down)
            pass
        elif temp == 3:
            temp = self.transform.angle - 270
            x = temp/90
            #print(x)
            y = 1-x
            self.transform.position=Vector.Move(self.transform.position,speed * y,Vector.up)
            self.transform.position=Vector.Move(self.transform.position,speed * x,Vector.right)
            pass
        #print(temp)
        pass


from GameObject.Vector2 import Vector
import pygame
from pygame.locals import *
from Engine.Engine import Engine
from Timer.Time import Time
import _thread
class Transform(pygame.sprite.Sprite):
    def __init__(self, image: str):
        pygame.sprite.Sprite.__init__(self)
        self.__agl=0.0
        self.__nowid=0
        self.__scl=Vector(100.0,100.0)
        self.position=Vector()
        self.image=pygame.image.load(image).convert_alpha()
        self.rect=self.image.get_rect()
        self.mask=pygame.mask.from_surface(self.image)
        self.tempimage=pygame.image.load(image).convert_alpha()
        self.drawId=0
        self.imageRect=self.image.get_rect()
        self.imageRect = self.imageRect.move((0,0))
        self.newRect=self.image.get_rect()
        self.newRect=self.newRect.move((900-self.newRect.width)/2,(600-self.newRect.height)/2)
        self.__drawL={}
        self.__drawL[self.drawId]=(self.image, self.tempimage)
        self.__wid=self.image.get_size()[0]
        self.__hgt=self.image.get_size()[1]
        self.parent=None
        self.position.parent=self
        #_thread.start_new_thread(self.UpdateRect,())
        pass
    def SetParent(self,parent):
        self.parent=parent
        pass
    def GetRect():
        pass
    def Add(self,image: str):
        self.drawId+=1
        self.__drawL[self.drawId]=[pygame.image.load(image).convert_alpha(), pygame.image.load(image).convert_alpha()]
        pass
    def Set(self,id: int):
        self.image=self.__drawL[id][0]
        self.__nowid=id
        self.tempimage=self.__drawL[id][1]
        
    @property
    def width(self):
        return self.__wid*self.__scl.x/100
    @property
    def height(self):
        return self.__hgt*self.__scl.y/100
    @property
    def angle(self):
        return self.__agl
    @property
    def localScale(self):
        return Vector(self.__scl.x,self.__scl.y)
    def Translate(self,x: float,y: float):
        self.__scl.x=x
        self.__scl.y=y
        temp=self.mask.get_size()
        self.mask=self.mask.scale((int(temp[0]*(x/100)),int(temp[1]*(y/100))))
        self.__UpdateLocalScale()
        temp2 = self.__drawL[self.__nowid]
        self.__drawL[self.__nowid]=[self.image,temp2[1]]
    def __UpdateLocalScale(self):
        self.image = pygame.transform.scale(self.image,(int(self.__wid*(self.__scl.x/100)),int(self.__hgt*(self.__scl.y/100))))
        pass
    @property
    def GetRect(self):
        return self.image.get_rect()
    def Rotate(self, angle: float):
        while angle < 0:
            angle+=360
        self.__agl+=angle
        if self.__agl>360:
            self.__agl-=360
        self.image = pygame.transform.rotate(self.tempimage,self.__agl)
        for x in range(0,self.drawId+1):
            temp2 = self.__drawL[x]
            self.__drawL[x]=[self.image,temp2[1]]
            pass
        self.newRect = self.image.get_rect(center=self.tempimage.get_rect().center)
        pass
    def Rotate2(self,angle: float):#亮向量夹角
        self.__agl =90- angle
        self.image = pygame.transform.rotate(self.tempimage,self.__agl)
        for x in range(0,self.drawId+1):
            temp2 = self.__drawL[x]
            self.__drawL[x]=[self.image,temp2[1]]
            pass
        self.newRect = self.image.get_rect(center=self.tempimage.get_rect().center)
        pass
    def LookAround(setf,obj):#向量间距离=r,圆周运动
        pass
    def UpdateRect(self):
        self.rect=pygame.Rect(self.position.x,self.position.y,self.width,self.height)
        pass


import _thread
from Timer.Time import Time
import pygame
import sys
import time as t
from Engine.Mathf import Mathf
from Engine.Collider import Collider
pygame.init()
 
class Engine(object):
    fps = 1/60
    drawlist={}
    drawSortList=[]
    screen=None
    time=None
    __isMouseDown_=False
    __isMouseUp_=True
    __isMouse=False
    colliderList=[]
    isDrawColliderRect=False
    isUseEngineCtrl=False
    @staticmethod
    def Start(screen=None):
        Engine.time=Time()
        #_thread.start_new_thread(Engine.CheckEvent,())
        if not screen==None:
            Engine.screen=screen
        pass
    @staticmethod
    def RegisterCollider(collider: Collider):
        Engine.colliderList.append(collider)
    @staticmethod
    def isMouseDown():
        return Engine.__isMouseDown_
        pass
    @staticmethod
    def isMouseClicked():
        if Engine.__isMouseDown_:
            Engine.__isMouse=True
        if Engine.__isMouseUp_:
            if Engine.__isMouse:
                Engine.__isMouse=False
                return True
        pass
    @staticmethod
    def CheckEvent():
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit()
            if event.type == pygame.MOUSEBUTTONDOWN:
                Engine.__isMouseDown_=True
                Engine.__isMouseUp_=False
            elif event.type == pygame.MOUSEBUTTONUP:
                Engine.__isMouseDown_=False 
                Engine.__isMouseUp_=True
    @staticmethod
    def isMouseUp():
        return Engine.__isMouseUp_
        pass
    @staticmethod
    def A():
        pass
    @staticmethod
    def SetMode(resolution=(0,0),flags=0,depth=0):
        Engine.screen=pygame.display.set_mode(resolution,flags,depth)
    @staticmethod
    def SetCaption(title: str, icontitle = None):
        if icontitle==None:
            pygame.display.set_caption(title)
        else:
            pygame.display.set_caption(title,icontitle)
    @staticmethod
    def Delete(obj):
        del Engine.drawlist[str(obj.priority)]
        Engine.drawSortList.remove(obj.priority)
        if obj.isMianCol:
            del obj.mycol
        if obj.isCol:
            for x in obj.ColList:
                x.Del(obj)
        del obj
        pass
    @staticmethod
    def Draw():
        Engine.time.CalculationDeltaTime()
        if Engine.isUseEngineCtrl:
            Engine.CheckEvent()
        #print(Engine.drawlist.keys())
        #print(Engine.drawSortList)
        for x in Engine.drawSortList:
            temp=Engine.drawlist[str(x)]
            if temp.active:
                temp.transform.UpdateRect()
                Engine.screen.blit(temp.transform.image,temp.transform.position.value,temp.transform.image.get_rect())
                #temp.transform.rect=temp.transform.rect.move(Mathf.TupSub(temp.transform.position.value,temp.transform.position.oldV))
                #Engine.screen.blit(temp.transform.image,temp.transform.position.value,temp.transform.image.get_rect(center=temp.transform.newRect.center))
                if Engine.isDrawColliderRect:
                    pygame.draw.rect(Engine.screen, (255, 0, 0), temp.transform.rect, 1)
        for i in Engine.colliderList:
            i.collision()
        
        pygame.display.update()

        


import math
from Timer.Time import Time
class Mathf(object):
    @staticmethod
    def Lerp(a: float, b: float, t: float):
        if t <= 0.0:
            return a;
        elif t >= 1.0:
            return b;
        return t * b + (1.0 - t) * a
    @staticmethod
    def PingPong(t: float, length: int):
        if int(t)%2==0:
            return t % length
        else:
            return length-(t % length)
        pass
    @staticmethod
    def Clamps(value: float, min: float, max: float):
        while value>=min and value<=max:
            if value < min:
                value += min
            elif value > max:
                value -= max
        return value
        pass
    @staticmethod
    def TupSub(tup1: tuple,tup2: tuple):
        #print(tup1,tup2)
        return(tup1[0]-tup2[0],tup1[1]-tup2[1])
    @staticmethod
    def MySort(A,B):
        if A>B:
            return 1
        elif A<B:
            return -1
        else:
            return 0



from GameObject.GameObject import GameObject
from Engine.Engine import Engine
from Timer.Time import Time
from GameObject.Vector2 import Vector
from Engine.Mathf import Mathf
from Engine.Collider import Collider





import pygame
class Collider:
    def __init__(self, obj):
        self.group=[]
        obj.isMianCol=True
        obj.mycol=self
        self.obj=obj.transform
        self.isCollision=False
        pass
    def Add(self, obj):
        obj.ColList.append(self.obj.parent.mycol)
        obj.isCol=True
        self.group.append(obj.transform)
        pass
    def Del(self, obj):
        obj.isCol=False
        for x in self.group:
            if x.parent.thisID == obj.thisID:
                self.group.remove(x)
        pass
    def collision(self):
        for x in self.group:
            if(pygame.sprite.collide_mask(self.obj,x)):
                #print(x.parent.priority,pygame.sprite.collide_mask(self.obj,x))
                self.isCollision=True
                return True
            else:
                self.isCollision=False

            #xoffset = self.obj.rect.x - x.rect.x
            #yoffset = self.obj.rect.y - x.rect.y

            #point = x.mask.overlap(self.obj.mask, (xoffset, yoffset))
            #someArea = x.mask.overlap_mask(self.obj.mask, (xoffset, yoffset))
            #if point:
                #px,py=point
                #cx,cy = px - x.rect.x, py - x.rect.y
                #pixel1=x.image.get_at((cx,cy))

                #qx,qy=px - self.obj.rect.x, py - self.obj.rect.y
                #pixel2=self.obj.image.get_at((qx,qy))

                #print(point,someArea,pixel1,pixel2)
        return False


from setuptools import setup
setup(
    name='mc_sprite',
    version='0.0.23',
    author='魔扣少儿编程_Hugn',
    author_email='',
    url ='https://www.coding4fun.com.cn/',
    install_requires=['pygame>=1.9.4'
                      ],
    packages = ['GameObject','Engine','Timer'],
    data_files=[('',['mc_sprite.py'])
                ],
    include_package_data = True, 
    zip_safe=False,
    )